@@ -3,26 +3,46 @@
 import logging
 import gevent
 
-__all__ = ['SocketIOError', 'SocketIOContext',
-           'socketio_manage']
+__all__ = ['SocketIOError', 'SocketIOContext', 'socketio_manage']
 
 log = logging.getLogger(__name__)
 
+
 class SocketIOError(Exception):
     pass
 
+
 class SocketIOKeyAssertError(SocketIOError):
     pass
 
+
+def require_connection(fn):
+    def wrapped(ctx, *args, **kwargs):
+        io = ctx.io
+
+        if not io.session.connected:
+            ctx.kill()
+            ctx.debug("not connected on %s: exiting greenlet", fn.__name__)
+            raise gevent.GreenletExit()
+
+        return fn(ctx, *args, **kwargs)
+
+    return wrapped
+
+
 class SocketIOContext(object):
-    def __init__(self, request, in_type="type", out_type="type", debug=False):
-        """Called when you create a new context, either by hand or from a nested context.
+    def __init__(self, request, in_type="type", out_type="type", debug=False,
+                 json_dumps=None, json_loads=None):
+        """Called when you create a new context, either by hand or from a
+           nested context.
 
         Arguments:
         * ``request`` - the pyramid request
         * ``in_type`` - the dict. key for message names of incoming messages
         * ``out_type`` - the dict. key for message names, in outgoing message
         * ``debug`` - whether to disable debug logging...
+        * ``json_dumps`` - the function to use instead of `json.dumps`
+        * ``json_loads`` - the function to use instead of `json.loads`
 
         On the object you subclass from this one, you should define methods
         using the "msg_message_type" naming convention, where 'message_type'
@@ -32,6 +52,10 @@ class SocketIOContext(object):
         """
         self.request = request
         self.io = request.environ['socketio']
+        if json_dumps:
+            self.io.dumps = json_dumps
+        if json_loads:
+            self.io.loads = json_loads
         self._parent = None
         self._in_type = in_type
         self._out_type = out_type
@@ -42,7 +66,7 @@ class SocketIOContext(object):
 
         # Override self.debug if in production mode
         if not debug:
-             self.debug = lambda x: None
+            self.debug = lambda x: None
 
     def debug(self, msg):
         print "%s: %s" % (self.id, msg)
@@ -80,6 +104,9 @@ class SocketIOContext(object):
         self.request = None
         self.io = None
 
+        if hasattr(self, 'disconnect'):
+            getattr(self, 'disconnect')()
+
         for callback, ar, kwar in self._on_disconnect:
             # NOTE: should we have a way to declare Blocking on_disconnect
             # callbacks, or should these things all be spawned to some other
@@ -97,7 +124,6 @@ class SocketIOContext(object):
             if io:
                 io.session.kill()
             return
-            
 
     def switch(self, new_context, *args, **kwargs):
         """Switch context, stack up contexts and pass on request.
@@ -134,16 +160,25 @@ class SocketIOContext(object):
         out.update(kwargs)
         self.send(out)
 
+    @require_connection
     def send(self, msg):
-        """Wrapper for self.io.send, to detect any sending problem and
-        call the destroy callbacks"""
-        io = self.io
-        if not io.connected():
-            self.kill()
-            self.debug("not connected on send(): exiting greenlet")
-            raise gevent.GreenletExit()
+        """ Sends a message to the socket """
         self.io.send(msg)
 
+    @require_connection
+    def send_event(self, name, msg):
+        """ Sends a custom event to the socket """
+        self.io.send_event(name, msg)
+
+    @require_connection
+    def broadcast(self, msg):
+        """ Broadcasts a message to all clients but this one """
+        self.io.broadcast(msg)
+
+    @require_connection
+    def broadcast_event(self, name, msg):
+        """ Broadcasts a custom event to all clients but this one """
+        self.io.broadcast_event(name, msg)
 
     def assert_keys(self, msg, elements):
         """Make sure the elements are inside the message, otherwise send an
@@ -161,55 +196,74 @@ class SocketIOContext(object):
         """Parse the message upon reception and dispatch it to the good method.
         """
         in_type = self._in_type
-        msg_type = "msg_" + msg[in_type]
+        msg_type = msg[in_type]
+
+        argval = None
+
+        if msg_type == "event":
+            msg_type += "_%s" % msg['name']
+
+            if 'args' in msg:
+                argval = msg['args']
+        else:
+            if 'data' in msg:
+                argval = msg['data']
+
         if not hasattr(self, msg_type) or \
                 not callable(getattr(self, msg_type)):
             self.error("unknown_command", "Command unknown: %s" % msg[in_type])
             return
         try:
             self.debug("Calling msg type: %s with obj: %s" % (msg_type, msg))
-            return getattr(self, msg_type)(msg)
+            return getattr(self, msg_type)(argval)
         except SocketIOKeyAssertError, e:
             return None
 
 
 def watcher(request):
-    """Watch if any of the greenlets for a request have died. If so, kill the request and the socket.
+    """Watch if any of the greenlets for a request have died. If so, kill the
+       request and the socket.
     """
     # TODO: add that if any of the request.jobs die, kill them all and exit
     io = request.environ['socketio']
     gevent.sleep(5.0)
     while True:
         gevent.sleep(1.0)
-        if not io.connected():
+        if not io.session.connected:
             # TODO: Warning, what about the on_disconnect callbacks ?
             gevent.killall(request.jobs)
             return
 
+
 def socketio_recv(context):
     """Manage messages arriving from Socket.IO, dispatch to context handler"""
     io = context.io
     in_type = context._in_type
+
     while True:
-        for msg in io.recv():
+        message = io.receive()
+
+        if message:
             # Skip invalid messages
-            if not isinstance(msg, dict):
+            if not isinstance(message, dict):
                 context.error("bad_request",
-                              "Your message needs to be JSON-formatted")
-            elif in_type not in msg:
+                            "Your message needs to be JSON-formatted")
+            elif in_type not in message:
                 context.error("bad_request",
-                              "You need a 'type' attribute in your message")
+                            "You need a 'type' attribute in your message")
             else:
                 # Call msg in context.
-                newctx = context(msg)
+                newctx = context(message)
 
                 # Switch context ?
                 if newctx:
                     context = newctx
 
-        if not io.connected():
+        if not io.session.connected:
+            context.kill()
             return
 
+
 def socketio_manage(start_context):
     """Main SocketIO management function, call from within your Pyramid view.
 
@@ -219,19 +273,17 @@ def socketio_manage(start_context):
     request = start_context.request
     io = request.environ['socketio']
 
-    if not io.connected():
-        # probably asked for something else dude!
-        return "there's no reason to get here, you won't get any further. have you mapped socket.io/lib to something ?"
+    if hasattr(start_context, 'connect'):
+        getattr(start_context, 'connect')()
 
     # Run startup if there's one
-
     start_context.spawn(socketio_recv, start_context)
 
     # Launch the watcher thread
     killall = gevent.spawn(watcher, request)
 
     gevent.joinall(request.jobs + [killall])
-    
+
     start_context.debug("socketio_manage terminated")
 
     return "done"
